'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.fetchTableData = fetchTableData;
exports.resetExport = resetExport;

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

require('url-search-params-polyfill');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var instance = _axios2.default.create();
instance.defaults.timeout = 60000;

// fetchTableData - search for table data
function fetchTableData(props, tableSettings, limit, offset, sortName, sortOrder, searchValue, columnFilters, apiLocation) {
    return function (dispatch) {
        if (tableSettings.tableID) {
            if (limit === 1000) {
                var prevDataTableExportData = props.DataTableExportData ? _extends({}, props.DataTableExportData) : {};
                var existingTableDataData = null;
                var existingTableDataDataTotalSize = null;
                if (prevDataTableExportData && prevDataTableExportData[tableSettings.tableID]) {
                    existingTableDataData = prevDataTableExportData[tableSettings.tableID].data;
                    existingTableDataDataTotalSize = prevDataTableExportData[tableSettings.tableID].dataTotalSize;
                }
                var tableData = {};
                tableData[tableSettings.tableID] = {
                    fetching: true,
                    fetched: false,
                    error: null,
                    data: existingTableDataData,
                    dataTotalSize: existingTableDataDataTotalSize
                };
                var newDataTableData = _extends(prevDataTableExportData, tableData);

                dispatch({
                    type: 'FETCH_EXPORT_TABLE_DATA',
                    payload: newDataTableData
                });
            } else {
                var prevDataTableData = props.DataTableData ? _extends({}, props.DataTableData) : {};
                var _existingTableDataData = null;
                var _existingTableDataDataTotalSize = null;
                if (prevDataTableData && prevDataTableData[tableSettings.tableID]) {
                    _existingTableDataData = prevDataTableData[tableSettings.tableID].data;
                    _existingTableDataDataTotalSize = prevDataTableData[tableSettings.tableID].dataTotalSize;
                }
                var _tableData = {};
                _tableData[tableSettings.tableID] = {
                    fetching: true,
                    fetched: false,
                    error: null,
                    data: _existingTableDataData,
                    dataTotalSize: _existingTableDataDataTotalSize
                };
                var _newDataTableData = _extends(prevDataTableData, _tableData);

                dispatch({
                    type: 'FETCH_TABLE_DATA',
                    payload: _newDataTableData
                });
            }
            var params = new URLSearchParams();
            params.append('tableSettings', JSON.stringify(tableSettings));
            params.append('limit', limit);
            params.append('offset', offset);
            if (typeof sortName !== 'undefined') params.append('sortName', sortName);
            if (typeof sortOrder !== 'undefined') params.append('sortOrder', sortOrder);
            if (typeof searchValue !== 'undefined') params.append('searchValue', searchValue);
            if (typeof columnFilters !== 'undefined') params.append('columnFilters', JSON.stringify(columnFilters));
            instance.post(apiLocation, params, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'
                }
            }).then(function (response) {
                var prevDataTableExportData = props.DataTableExportData ? _extends({}, props.DataTableExportData) : {};
                var prevDataTableData = props.DataTableData ? _extends({}, props.DataTableData) : {};
                if (response.data.searchSuccess) {
                    var _tableData2 = {};
                    _tableData2[tableSettings.tableID] = {
                        fetching: false,
                        fetched: true,
                        error: null,
                        data: response.data.data,
                        dataTotalSize: response.data.dataTotalSize
                    };

                    if (limit === 1000) {
                        var _newDataTableData2 = _extends(prevDataTableExportData, _tableData2);
                        dispatch({
                            type: 'FETCH_EXPORT_TABLE_DATA_FULFILLED',
                            payload: _newDataTableData2
                        });
                    } else {
                        var _newDataTableData3 = _extends(prevDataTableData, _tableData2);
                        dispatch({
                            type: 'FETCH_TABLE_DATA_FULFILLED',
                            payload: _newDataTableData3
                        });
                    }
                } else {
                    var _tableData3 = {};
                    _tableData3[tableSettings.tableID] = {
                        fetching: false,
                        fetched: false,
                        error: response,
                        data: null,
                        dataTotalSize: 0
                    };

                    if (limit === 1000) {
                        var _newDataTableData4 = _extends(prevDataTableExportData, _tableData3);
                        dispatch({
                            type: 'FETCH_EXPORT_TABLE_DATA_REJECTED',
                            payload: _newDataTableData4
                        });
                    } else {
                        var _newDataTableData5 = _extends(prevDataTableData, _tableData3);
                        dispatch({
                            type: 'FETCH_TABLE_DATA_REJECTED',
                            payload: _newDataTableData5
                        });
                    }
                }
            }).catch(function (err) {
                var prevDataTableData = props.DataTableData ? _extends({}, props.DataTableData) : {};
                var tableData = {};
                tableData[tableSettings.tableID] = {
                    fetching: false,
                    fetched: false,
                    error: err,
                    data: null,
                    dataTotalSize: 0
                };
                var newDataTableData = _extends(prevDataTableData, tableData);

                dispatch({
                    type: 'FETCH_TABLE_DATA_REJECTED',
                    payload: newDataTableData
                });
            });
        }
    };
}

// resetExport - reset the export
function resetExport(props, tableID) {
    return function (dispatch) {
        var prevDataTableExportData = props.DataTableExportData ? _extends({}, props.DataTableExportData) : {};
        var tableData = {};
        tableData[tableID] = {
            fetching: false,
            fetched: false,
            error: null,
            data: null,
            dataTotalSize: 0
        };
        var newDataTableData = _extends(prevDataTableExportData, tableData);
        dispatch({ type: 'FETCH_EXPORT_TABLE_DATA_RESET', payload: newDataTableData });
    };
}