'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactRedux = require('react-redux');

var _DataTable = require('./DataTable');

var _DataTable2 = _interopRequireDefault(_DataTable);

var _DataTable3 = require('./DataTable.actions');

var _LoadingGif = require('./LoadingGif/LoadingGif');

var _LoadingGif2 = _interopRequireDefault(_LoadingGif);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function setFiltersFromStorage(tableColumns, previousFilters) {
    var updatedTableColumns = [];
    if (Object.prototype.toString.call(tableColumns) === '[object Array]') {
        tableColumns.forEach(function (col) {
            var type = col.filter ? col.filter : 'TextFilter';
            if (previousFilters && previousFilters[col.key] && previousFilters[col.key].value) {
                if (type === 'CustomDateRangeFilter') {
                    var fromDate = new Date(previousFilters[col.key].value.values.from);
                    var toDate = new Date(previousFilters[col.key].value.values.to);
                    updatedTableColumns.push(_extends({}, col, {
                        defaultValue: {
                            from: fromDate,
                            to: toDate
                        } // replace default
                    }));
                } else {
                    updatedTableColumns.push(_extends({}, col, {
                        defaultValue: previousFilters[col.key].value // replace default
                    }));
                }
            } else if (previousFilters) {
                if (type === 'NumberFilter') {
                    updatedTableColumns.push(_extends({}, col, {
                        defaultValue: {
                            comparator: col.defaultValue.comparator
                        } // keep default comparator
                    }));
                } else {
                    var newCol = _extends({}, col);
                    delete newCol.defaultValue; // remove any defaults
                    updatedTableColumns.push(_extends({}, newCol));
                }
            } else {
                updatedTableColumns.push(_extends({}, col)); // no change
            }
        });
    }
    return updatedTableColumns;
}

function generateFilterObj(tableColumns) {
    var filterObj = {};
    if (Object.prototype.toString.call(tableColumns) === '[object Array]') {
        tableColumns.forEach(function (col) {
            if (col.defaultValue) {
                var type = col.filter ? col.filter : 'TextFilter';
                if (type === 'NumberFilter') {
                    if (col.defaultValue.number) {
                        filterObj[col.key] = {
                            type: col.filter,
                            value: col.defaultValue
                        };
                    }
                } else {
                    var value = '';
                    if (type === 'CustomDateRangeFilter') {
                        type = 'CustomFilter';
                        value = {
                            key: 'value',
                            type: 'between',
                            value: col.defaultValue
                        };
                    } else {
                        value = col.defaultValue;
                    }
                    filterObj[col.key] = {
                        type: type,
                        value: value
                    };
                }
            }
        });
    }
    return filterObj;
}

function generateColumnFilters(filterObj) {
    var columnFilters = [];
    if (Object.keys(filterObj).length > 0) {
        for (var key in filterObj) {
            if (filterObj[key].type) {
                switch (filterObj[key].type) {
                    case 'TextFilter':
                        {
                            columnFilters.push({
                                key: key,
                                type: 'like',
                                value: filterObj[key].value
                            });
                            break;
                        }
                    case 'SelectFilter':
                        {
                            columnFilters.push({
                                key: key,
                                type: 'eq',
                                value: filterObj[key].value
                            });
                            break;
                        }
                    case 'NumberFilter':
                        {
                            var type = 'like';
                            switch (filterObj[key].value.comparator) {
                                case '=':
                                    type = 'eq';
                                    break;
                                case '>':
                                    type = 'gt';
                                    break;
                                case '>=':
                                    type = 'gteq';
                                    break;
                                case '<':
                                    type = 'lt';
                                    break;
                                case '<=':
                                    type = 'lteq';
                                    break;
                                case '!=':
                                    type = 'nteq';
                                    break;
                                default:
                            }
                            columnFilters.push({
                                key: key,
                                type: type,
                                value: filterObj[key].value.number
                            });
                            break;
                        }
                    case 'CustomFilter':
                        {
                            var filterValue = '';
                            if (typeof filterObj[key].value.values !== 'undefined') {
                                filterValue = filterObj[key].value.values;
                            } else if (typeof filterObj[key].value.value !== 'undefined') {
                                filterValue = filterObj[key].value.value;
                            }
                            columnFilters.push({
                                key: key,
                                type: filterObj[key].value.type,
                                value: filterValue
                            });
                            break;
                        }
                    default:
                }
            }
        }
    }
    return columnFilters;
}

var DataTableContainer = function (_React$Component) {
    _inherits(DataTableContainer, _React$Component);

    function DataTableContainer(props) {
        _classCallCheck(this, DataTableContainer);

        var _this = _possibleConstructorReturn(this, (DataTableContainer.__proto__ || Object.getPrototypeOf(DataTableContainer)).call(this, props));

        _this.makeFullscreen = _this.makeFullscreen.bind(_this);
        _this.onPageChange = _this.onPageChange.bind(_this);
        _this.onSizePerPageList = _this.onSizePerPageList.bind(_this);
        _this.onSortChange = _this.onSortChange.bind(_this);
        _this.onSearchChange = _this.onSearchChange.bind(_this);
        _this.onFilterChange = _this.onFilterChange.bind(_this);
        _this.onExportToCSV = _this.onExportToCSV.bind(_this);
        _this.resetPagination = _this.resetPagination.bind(_this);
        _this.startClearingFilters = _this.startClearingFilters.bind(_this);
        _this.clearFilters = _this.clearFilters.bind(_this);
        _this.initiateTable = _this.initiateTable.bind(_this);
        _this.refreshTable = _this.refreshTable.bind(_this);
        _this.state = {
            isFullscreen: false,
            sizePerPage: 10,
            currentPage: 1,
            sortName: undefined,
            sortOrder: undefined,
            searchValue: '' + (_this.props.tableSettings.defaultSearch ? _this.props.tableSettings.defaultSearch : ''),
            columnFilters: undefined,
            clearingFilters: false,
            filtersPristine: true,
            initReady: false,
            tableColumns: _this.props.tableSettings.tableColumns
        };
        return _this;
    }

    _createClass(DataTableContainer, [{
        key: 'componentWillMount',
        value: function componentWillMount() {
            if (this.props.tableSettings.useLocalStorage) {
                // set table search
                var previousTableSearch = JSON.parse(localStorage.getItem('tableSearch'));
                if (previousTableSearch && previousTableSearch[this.props.tableSettings.tableID]) {
                    var updatedTableSearch = previousTableSearch[this.props.tableSettings.tableID];
                    this.setState({
                        searchValue: updatedTableSearch
                    });
                }

                // set table filters
                var previousTableFilters = JSON.parse(localStorage.getItem('tableFilters'));
                if (previousTableFilters && previousTableFilters[this.props.tableSettings.tableID]) {
                    var updatedTableColumns = setFiltersFromStorage(this.props.tableSettings.tableColumns, previousTableFilters[this.props.tableSettings.tableID]);

                    this.setState({
                        tableColumns: updatedTableColumns
                    });
                }
            }
        }
    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.initiateTable();
        }
    }, {
        key: 'onFilterChange',
        value: function onFilterChange(filterObj) {
            if (this.state.initReady) {
                this.setState({
                    filtersPristine: false
                });
                var columnFilters = generateColumnFilters(filterObj);

                // check: do not update individually while clearing all filters
                if (!this.state.clearingFilters) {
                    if (this.props.tableSettings.useLocalStorage) {
                        var previousTableFilters = JSON.parse(localStorage.getItem('tableFilters'));
                        var newTableFilters = void 0;
                        if (previousTableFilters) {
                            newTableFilters = _extends({}, previousTableFilters, _defineProperty({}, this.props.tableSettings.tableID, filterObj));
                        } else {
                            newTableFilters = _defineProperty({}, this.props.tableSettings.tableID, filterObj);
                        }
                        localStorage.setItem('tableFilters', JSON.stringify(newTableFilters));
                    }
                    this.resetPagination();
                    this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, this.state.sizePerPage, 0, this.state.sortName, this.state.sortOrder, this.state.searchValue, columnFilters, this.props.apiLocation));
                    this.setState({
                        columnFilters: columnFilters
                    });
                }
            }
        }
    }, {
        key: 'onSearchChange',
        value: function onSearchChange(e) {
            this.resetPagination();
            var text = e.target.value.trim();
            if (this.props.tableSettings.useLocalStorage) {
                var previousTableSearch = JSON.parse(localStorage.getItem('tableSearch'));
                var newTableSearch = void 0;
                if (previousTableSearch) {
                    newTableSearch = _extends({}, previousTableSearch, _defineProperty({}, this.props.tableSettings.tableID, text));
                } else {
                    newTableSearch = _defineProperty({}, this.props.tableSettings.tableID, text);
                }
                localStorage.setItem('tableSearch', JSON.stringify(newTableSearch));
            }
            this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, this.state.sizePerPage, 0, this.state.sortName, this.state.sortOrder, text, this.state.columnFilters, this.props.apiLocation));
            this.setState({
                searchValue: text
            });
        }
    }, {
        key: 'onSortChange',
        value: function onSortChange(sortName, sortOrder) {
            var offset = (this.state.currentPage - 1) * this.state.sizePerPage;
            this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, this.state.sizePerPage, offset, sortName, sortOrder, this.state.searchValue, this.state.columnFilters, this.props.apiLocation));
            this.setState({
                sortName: sortName,
                sortOrder: sortOrder
            });
        }
    }, {
        key: 'onPageChange',
        value: function onPageChange(page, sizePerPage) {
            var offset = (page - 1) * sizePerPage;
            this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, sizePerPage, offset, this.state.sortName, this.state.sortOrder, this.state.searchValue, this.state.columnFilters, this.props.apiLocation));
            this.setState({
                currentPage: page
            });
        }
    }, {
        key: 'onSizePerPageList',
        value: function onSizePerPageList(sizePerPage) {
            this.setState({
                sizePerPage: sizePerPage
            });
        }
    }, {
        key: 'onExportToCSV',
        value: function onExportToCSV() {
            var tableID = this.props.tableSettings.tableID;
            if (this.props.DataTableExportData && this.props.DataTableExportData[tableID] && this.props.DataTableExportData[tableID].fetched) {
                var exportData = this.props.DataTableExportData[tableID].data;
                this.props.dispatch((0, _DataTable3.resetExport)(this.props, tableID));
                return exportData;
            } else if (!this.props.DataTableExportData || !this.props.DataTableExportData[tableID] || !this.props.DataTableExportData[tableID].fetching) {
                this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, 1000, 0, this.state.sortName, this.state.sortOrder, this.state.searchValue, this.state.columnFilters, this.props.apiLocation));
            }
            return false;
        }
    }, {
        key: 'initiateTable',
        value: function initiateTable() {
            var offset = (this.state.currentPage - 1) * this.state.sizePerPage;
            var filterObj = generateFilterObj(this.state.tableColumns);
            var columnFilters = generateColumnFilters(filterObj);
            this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, this.state.sizePerPage, offset, this.state.sortName, this.state.sortOrder, this.state.searchValue, columnFilters, this.props.apiLocation));
            this.setState({
                initReady: true,
                columnFilters: columnFilters
            });
        }
    }, {
        key: 'refreshTable',
        value: function refreshTable() {
            var offset = (this.state.currentPage - 1) * this.state.sizePerPage;
            this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, this.state.sizePerPage, offset, this.state.sortName, this.state.sortOrder, this.state.searchValue, this.state.columnFilters, this.props.apiLocation));
        }
    }, {
        key: 'startClearingFilters',
        value: function startClearingFilters() {
            this.setState({
                clearingFilters: true
            });
        }
    }, {
        key: 'clearFilters',
        value: function clearFilters() {
            if (this.props.tableSettings.useLocalStorage) {
                var previousTableFilters = JSON.parse(localStorage.getItem('tableFilters'));
                var newTableFilters = void 0;
                if (previousTableFilters) {
                    newTableFilters = _extends({}, previousTableFilters, _defineProperty({}, this.props.tableSettings.tableID, {}));
                } else {
                    newTableFilters = _defineProperty({}, this.props.tableSettings.tableID, {});
                }
                localStorage.setItem('tableFilters', JSON.stringify(newTableFilters));
            }
            this.props.dispatch((0, _DataTable3.fetchTableData)(this.props, this.props.tableSettings, this.state.sizePerPage, 0, this.state.sortName, this.state.sortOrder, this.state.searchValue, undefined, this.props.apiLocation));
            this.setState({
                currentPage: 1,
                columnFilters: undefined,
                clearingFilters: false
            });
        }
    }, {
        key: 'resetPagination',
        value: function resetPagination() {
            this.setState({
                currentPage: 1
            });
        }
    }, {
        key: 'makeFullscreen',
        value: function makeFullscreen() {
            this.setState({ isFullscreen: !this.state.isFullscreen });
        }
    }, {
        key: 'render',
        value: function render() {
            var _props = this.props,
                tableSettings = _props.tableSettings,
                DataTableData = _props.DataTableData,
                DataTableExportData = _props.DataTableExportData;


            var minWidth = tableSettings.minWidth;
            var tableID = tableSettings.tableID;

            if (!tableID || DataTableData && DataTableData[tableID] && DataTableData[tableID].error) {
                return _react2.default.createElement(
                    'div',
                    { className: 'applicationError status_message offline' },
                    _react2.default.createElement(
                        'p',
                        null,
                        'The table failed to initialise. Please check you are connected to the internet and try again.'
                    )
                );
            }

            var isLoading = false;
            if (!DataTableData || !DataTableData[tableID] || !DataTableData[tableID].fetched) isLoading = true;

            var tableData = null;
            if (DataTableData && DataTableData[tableID] && DataTableData[tableID].data) {
                tableData = DataTableData[tableID].data;
            }

            var tableDataSize = 0;
            if (DataTableData && DataTableData[tableID] && DataTableData[tableID].dataTotalSize) {
                tableDataSize = DataTableData[tableID].dataTotalSize;
            }

            if (!tableSettings.extraToolbarItems) tableSettings.extraToolbarItems = null;

            return _react2.default.createElement(
                'div',
                {
                    className: '\n                  ' + tableSettings.wrapperType + '\n                  ' + (this.state.isFullscreen ? 'section-isFullscreen' : '') + '\n                  react-datatable\n              '
                },
                tableSettings.displayTitle && _react2.default.createElement(
                    'div',
                    { className: 'section-toolbar' },
                    _react2.default.createElement(
                        'span',
                        { className: 'section-toolbar-title' },
                        tableSettings.displayTitle
                    ),
                    tableSettings.extraToolbarItems && tableSettings.extraToolbarItems(),
                    _react2.default.createElement(
                        'div',
                        { className: 'section-toolbar-group' },
                        _react2.default.createElement(
                            'div',
                            {
                                className: '\n                              section-toolbar-icon\n                              section-toolbar-fullscreen\n                              ' + (this.state.isFullscreen ? 'section-toolbar-isFullscreen' : '') + '\n                          ',
                                title: 'Toggle Fullscreen',
                                onClick: this.makeFullscreen
                            },
                            'Fullscreen'
                        )
                    )
                ),
                _react2.default.createElement(
                    'div',
                    { className: 'inner', style: { overflow: 'auto' } },
                    _react2.default.createElement(
                        'div',
                        { style: { minWidth: minWidth } },
                        isLoading && _react2.default.createElement(
                            'div',
                            { className: 'absLoadingContainer', style: { opacity: 0.3 } },
                            _react2.default.createElement(_LoadingGif2.default, null)
                        ),
                        _react2.default.createElement(_DataTable2.default, {
                            keyField: tableSettings.keyField,
                            extraButtons: tableSettings.extraButtons,
                            defaultSort: tableSettings.defaultSort,
                            tableColumns: this.state.tableColumns,
                            tableData: tableData,
                            DataTableExportData: DataTableExportData[tableID],
                            dataTotalSize: tableDataSize,
                            onPageChange: this.onPageChange,
                            onSizePerPageList: this.onSizePerPageList,
                            onSortChange: this.onSortChange,
                            onSearchChange: this.onSearchChange,
                            onFilterChange: this.onFilterChange,
                            onExportToCSV: this.onExportToCSV,
                            currentPage: this.state.currentPage,
                            sizePerPage: this.state.sizePerPage,
                            refreshTable: this.refreshTable,
                            sortName: this.state.sortName,
                            sortOrder: this.state.sortOrder,
                            searchValue: this.state.searchValue,
                            columnFilters: this.state.columnFilters,
                            startClearingFilters: this.startClearingFilters,
                            clearFilters: this.clearFilters,
                            filtersPristine: this.state.filtersPristine
                        })
                    )
                )
            );
        }
    }]);

    return DataTableContainer;
}(_react2.default.Component);

DataTableContainer.propTypes = {
    dispatch: _propTypes2.default.func.isRequired,
    tableSettings: _propTypes2.default.object.isRequired,
    apiLocation: _propTypes2.default.string.isRequired,
    DataTableData: _propTypes2.default.any,
    DataTableExportData: _propTypes2.default.object
};

DataTableContainer.defaultProps = {
    DataTableExportData: null,
    DataTableData: null
};

exports.default = (0, _reactRedux.connect)(function (store) {
    return {
        DataTableData: store.DataTableReducer.DataTableData,
        DataTableExportData: store.DataTableExportReducer.DataTableExportData
    };
})(DataTableContainer);